#include "lights/spot-light.h"

using namespace imaging;
using namespace math;
using namespace raytracer;

namespace
{
    /// <summary>
    /// Implementation for spot lights.
    /// </summary>
    class SpotLight : public raytracer::lights::_private_::PointLightImplementation
    {
    public:
        SpotLight(const Point3D& position, const Vector3D& direction, const Angle& angle, const Color& color)
            : PointLightImplementation(position), m_direction(direction.normalized()), m_angle(angle), m_color(color) {}

    protected:
        LightRay cast_lightray_to(const math::Point3D& p) const override
        {
            // Create ray starting at the light source's position and going through p
            math::Ray ray(m_position, p);

            if ((p - m_position).normalized().dot(m_direction) >= cos(m_angle/2) ) {
                return LightRay(ray, m_color);
            } else {
                return LightRay(ray, colors::black());
            }
        }

    private:
        math::Vector3D m_direction;
        math::Angle m_angle;
        imaging::Color m_color;
    };
}

LightSource raytracer::lights::spot(const Point3D& position, const Vector3D& direction, const Angle& angle, const Color& color)
{
    return LightSource(std::make_shared<SpotLight>(position, direction, angle, color));
}

LightSource raytracer::lights::spot(const Point3D& position, const Point3D& look_at, const Angle& angle, const Color& color)
{
    return LightSource(std::make_shared<SpotLight>(position, look_at-position, angle, color));
}