#include "boundingbox-primitive.h"
#include <assert.h>

using namespace raytracer;
using namespace math;

namespace
{
    class BoundingBoxImplementation : public raytracer::primitives::_private_::PrimitiveImplementation
    {
    public:
        BoundingBoxImplementation(const Primitive& p) : m_primitive(p), m_box(p->bounding_box()) {}

        Box bounding_box() const override
        {
            return m_box;
        }

        std::vector<std::shared_ptr<Hit>> find_all_hits(const Ray& ray) const override
        {
            if (m_box.is_hit_by(ray))
            {
                return m_primitive->find_all_hits(ray);
            }
            return std::vector<std::shared_ptr<Hit>>();
        }

        void initialize_hit(Hit* hit, const Ray& ray, double t) const
        {
            hit->t = t;
            hit->position = ray.at(hit->t);
            hit->local_position.xyz = hit->position;
            hit->local_position.uv = Point2D(hit->position.x(), hit->position.y());
        }

    protected:
        const Box m_box;
        const Primitive m_primitive;
    };
}

Primitive raytracer::primitives::bounding_box_accelerator(const Primitive& p)
{
    return Primitive(std::make_shared<BoundingBoxImplementation>(p));
}
