#include "raytracers/ray-tracer-v3.h"
#include "easylogging++.h"
#include <stdlib.h> 

using namespace imaging;
using namespace math;
using namespace raytracer;

Color raytracer::raytracers::_private_::RayTracerV3::process_light_ray(const Scene& scene, const MaterialProperties& matProp, const Hit& hit, const math::Ray& ray, const LightRay& lightray) const
{
	auto result = process_light_ray(scene, matProp, hit, ray, lightray);
	result += compute_specular(matProp, hit, ray, lightray);
	return result;
}

Color raytracer::raytracers::_private_::RayTracerV3::compute_specular(const MaterialProperties& matProp, const Hit& hit, const math::Ray& ray, const LightRay& lightray) const
{
	Color result = colors::black();
	auto dirIncomingLight = (hit.position - lightray.ray.origin).normalized();
	auto reflectedDir = dirIncomingLight.reflect_by(dirIncomingLight);
	auto unitVector =  (ray.origin - hit.position).normalized();
	auto cosAlpha = unitVector.dot(reflectedDir);

	if (cosAlpha > 0) {
		result += lightray.color * matProp.specular * pow(cosAlpha,matProp.specular_exponent);
	}
	return result;
	
}

RayTracer raytracer::raytracers::v3()
{
	return raytracer::RayTracer(std::make_shared<_private_::RayTracerV3>());
}
