#include "samplers/multijittered-sampler.h"
#include "math/rasterizer.h"
#include <random>

using namespace math;
using namespace raytracer;

namespace
{
    class MultijitteredSampler : public raytracer::samplers::_private_::SamplerImplementation
    {
    public:
        MultijitteredSampler(const int n) : m_n(n), rd(), gen(rd()), dis(0.0, 1.0) {
            for (int i = 0; i < n; i++)
            {
                x_lijst.push_back(i);
            }
            y_lijst = x_lijst;
        }

        void sample(const math::Rectangle2D& rectangle, std::function<void(const math::Point2D&)> function) const override
        {
            math::Rasterizer rasterizer = math::Rasterizer(rectangle, m_n, m_n);

            std::shuffle(x_lijst.begin(), x_lijst.end(), rd);
            
            // for each y
            for (int y = 0; y < m_n; y++)
            {            
                std::shuffle(y_lijst.begin(), y_lijst.end(), rd);

                // for each x
                for (int x = 0; x < m_n; x++)
                {
                    math::Rasterizer sub_rasterizer = math::Rasterizer(rasterizer[Position2D(x, y)], m_n, m_n);

                    function(sub_rasterizer[Position2D(x_lijst.at(y), y_lijst.at(x))].from_relative(Point2D(dis(gen), dis(gen))));
                }
            }
        }
    private:
        int m_n;
        mutable std::vector<int> x_lijst;
        mutable std::vector<int> y_lijst;

        mutable std::random_device rd;
        mutable std::mt19937 gen;
        mutable std::uniform_real_distribution<double> dis;
    };
}

Sampler raytracer::samplers::multijittered(const int n)
{
    return Sampler(std::make_shared<MultijitteredSampler>(n));
}
