#include "samplers/n-rooks-sampler.h"
#include "math/rasterizer.h"
#include <random>

using namespace math;
using namespace raytracer;

namespace
{
    class NRooksSampler : public raytracer::samplers::_private_::SamplerImplementation
    {
    public:
        NRooksSampler(const int n) : m_n(n), rd(), gen(rd()), dis(0.0, 1.0) {
            for (int i = 0; i < n; i++)
            {
                lijst.push_back(i);
            }
        }

        void sample(const math::Rectangle2D& rectangle, std::function<void(const math::Point2D&)> function) const override
        {
            math::Rasterizer rasterizer = math::Rasterizer(rectangle, m_n, m_n);
            std::shuffle(lijst.begin(), lijst.end(), rd);

            //for each y choose a random x between 0 and N
            for (int i = 0; i < m_n; i++)
            {
                function(rasterizer[Position2D(lijst.at(i), i)].from_relative(Point2D(dis(gen), dis(gen))));
            }
        }
    private:
        int m_n;
        mutable std::vector<int> lijst;

        mutable std::random_device rd;
        mutable std::mt19937 gen;
        mutable std::uniform_real_distribution<double> dis;
    };
}

Sampler raytracer::samplers::n_rooks(const int n)
{
    return Sampler(std::make_shared<NRooksSampler>(n));
}
