#include "samplers/random-sampler.h"
#include <random>

using namespace math;
using namespace raytracer;

namespace
{
    class RandomSampler : public raytracer::samplers::_private_::SamplerImplementation
    {
    public:
        RandomSampler(const int sample_count) : m_sample_count(sample_count), rd(), gen(rd()), dis(0.0, 1.0) {}

        void sample(const math::Rectangle2D& rectangle, std::function<void(const math::Point2D&)> function) const override
        {
            for (int n = 0; n < m_sample_count; ++n) {
                function(rectangle.from_relative(Point2D(dis(gen), dis(gen))));
            }
        }
    private:
        int m_sample_count;

        mutable std::random_device rd;
        mutable std::mt19937 gen;
        mutable std::uniform_real_distribution<double> dis;
    };
}

Sampler raytracer::samplers::random(const int sample_count)
{
    return Sampler(std::make_shared<RandomSampler>(sample_count));
}