#include "samplers/stratified-sampler.h"
#include "math/rasterizer.h"
#include <random>

using namespace math;
using namespace raytracer;

namespace
{
    class StratifiedSampler : public raytracer::samplers::_private_::SamplerImplementation
    {
    public:
        StratifiedSampler(const int n, const int m) : m_n(n), m_m(m) {}

        void sample(const math::Rectangle2D& rectangle, std::function<void(const math::Point2D&)> function) const override
        {
            math::Rasterizer rasterizer = math::Rasterizer(rectangle, m_m, m_n);
            
            for (int m = 0; m < m_m; m++) {
                for (int n = 0; n < m_n; n++) {
                    function(rasterizer[Position2D(m, n)].center());
                }
            }
        }
    private:
        int m_n;
        int m_m;
    };
}

Sampler raytracer::samplers::stratified(const int n, const int m)
{
    return Sampler(std::make_shared<StratifiedSampler>(n, m));
}
